@@ -565,7 +565,7 @@ static int add_partition_root_rw(DissectedPartition *p) {
         path = strjoina(arg_dest, "/systemd-remount-fs.service.d/50-remount-rw.conf");
 
         r = write_string_file(path,
-                              "# Automatically generated by systemd-gpt-generator\n\n"
+                              "# Automatically generated by systemd-gpt-auto-generator\n\n"
                               "[Service]\n"
                               "Environment=SYSTEMD_REMOUNT_ROOT_RW=1\n",
                               WRITE_STRING_FILE_CREATE|WRITE_STRING_FILE_NOFOLLOW|WRITE_STRING_FILE_MKDIR_0755);
