@@ -48,10 +48,7 @@ int clock_set_hwclock(const struct tm *tm) {
         if (fd < 0)
                 return -errno;
 
-        if (ioctl(fd, RTC_SET_TIME, tm) < 0)
-                return -errno;
-
-        return 0;
+        return RET_NERRNO(ioctl(fd, RTC_SET_TIME, tm));
 }
 
 int clock_is_localtime(const char* adjtime_path) {
@@ -131,18 +128,19 @@ int clock_reset_timewarp(void) {
 
         /* The very first call to settimeofday() does time warp magic. Do a dummy call here, so the time
          * warping is sealed and all later calls behave as expected. */
-        if (settimeofday(NULL, &tz) < 0)
-                return -errno;
-
-        return 0;
+        return RET_NERRNO(settimeofday(NULL, &tz));
 }
 
 #define EPOCH_FILE "/usr/lib/clock-epoch"
 
-int clock_apply_epoch(void) {
+int clock_apply_epoch(ClockChangeDirection *ret_attempted_change) {
+        usec_t epoch_usec, now_usec;
         struct stat st;
-        struct timespec ts;
-        usec_t epoch_usec;
+
+        /* NB: we update *ret_attempted_change in *all* cases, both
+         * on success and failure, to indicate what we intended to do! */
+
+        assert(ret_attempted_change);
 
         if (stat(EPOCH_FILE, &st) < 0) {
                 if (errno != ENOENT)
@@ -152,10 +150,17 @@ int clock_apply_epoch(void) {
         } else
                 epoch_usec = timespec_load(&st.st_mtim);
 
-        if (now(CLOCK_REALTIME) >= epoch_usec)
+        now_usec = now(CLOCK_REALTIME);
+        if (now_usec < epoch_usec)
+                *ret_attempted_change = CLOCK_CHANGE_FORWARD;
+        else if (CLOCK_VALID_RANGE_USEC_MAX > 0 && now_usec > usec_add(epoch_usec, CLOCK_VALID_RANGE_USEC_MAX))
+                *ret_attempted_change = CLOCK_CHANGE_BACKWARD;
+        else {
+                *ret_attempted_change = CLOCK_CHANGE_NOOP;
                 return 0;
+        }
 
-        if (clock_settime(CLOCK_REALTIME, timespec_store(&ts, epoch_usec)) < 0)
+        if (clock_settime(CLOCK_REALTIME, TIMESPEC_STORE(epoch_usec)) < 0)
                 return -errno;
 
         return 1;
