@@ -3,6 +3,7 @@
 
 #include <stdio.h>
 
+#include "macro.h"
 #include "main-func.h"
 
 int generator_open_unit_file(
@@ -42,6 +43,18 @@ int generator_write_cryptsetup_service_section(
                 const char *password,
                 const char *options);
 
+int generator_write_veritysetup_unit_section(
+                FILE *f,
+                const char *source);
+
+int generator_write_veritysetup_service_section(
+                FILE *f,
+                const char *name,
+                const char *data_what,
+                const char *hash_what,
+                const char *roothash,
+                const char *options);
+
 int generator_write_device_deps(
         const char *dir,
         const char *what,
@@ -74,11 +87,11 @@ void log_setup_generator(void);
         _DEFINE_MAIN_FUNCTION(                                          \
                 ({                                                      \
                         log_setup_generator();                          \
-                        if (argc > 1 && argc != 4)                      \
+                        if (!IN_SET(argc, 2, 4))                        \
                                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL), \
-                                                       "This program takes zero or three arguments."); \
+                                                       "This program takes one or three arguments."); \
                 }),                                                     \
-                impl(argc > 1 ? argv[1] : "/tmp",                       \
-                     argc > 1 ? argv[2] : "/tmp",                       \
-                     argc > 1 ? argv[3] : "/tmp"),                      \
+                impl(argv[1],                                           \
+                     argv[argc == 4 ? 2 : 1],                           \
+                     argv[argc == 4 ? 3 : 1]),                          \
                 r < 0 ? EXIT_FAILURE : EXIT_SUCCESS)
