@@ -1,4 +1,4 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
-int kill_unit(int argc, char *argv[], void *userdata);
+int verb_kill(int argc, char *argv[], void *userdata);
