@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <errno.h>
-#include <linux/loadavg.h>
 #include <locale.h>
 #include <math.h>
 #include <sys/socket.h>
@@ -11,8 +10,9 @@
 #include "log.h"
 #include "parse-util.h"
 #include "string-util.h"
+#include "tests.h"
 
-static void test_parse_boolean(void) {
+TEST(parse_boolean) {
         assert_se(parse_boolean("1") == 1);
         assert_se(parse_boolean("y") == 1);
         assert_se(parse_boolean("Y") == 1);
@@ -38,7 +38,7 @@ static void test_parse_boolean(void) {
         assert_se(parse_boolean("full") < 0);
 }
 
-static void test_parse_pid(void) {
+TEST(parse_pid) {
         int r;
         pid_t pid;
 
@@ -72,7 +72,7 @@ static void test_parse_pid(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_parse_mode(void) {
+TEST(parse_mode) {
         mode_t m;
 
         assert_se(parse_mode("-1", &m) < 0);
@@ -94,7 +94,7 @@ static void test_parse_mode(void) {
         assert_se(parse_mode(" 1", &m) >= 0 && m == 1);
 }
 
-static void test_parse_size(void) {
+TEST(parse_size_iec) {
         uint64_t bytes;
 
         assert_se(parse_size("", 1024, &bytes) == -EINVAL);
@@ -164,7 +164,77 @@ static void test_parse_size(void) {
         assert_se(parse_size("-10B 20K", 1024, &bytes) == -ERANGE);
 }
 
-static void test_parse_range(void) {
+TEST(parse_size_si) {
+        uint64_t bytes;
+
+        assert_se(parse_size("", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size("111", 1000, &bytes) == 0);
+        assert_se(bytes == 111);
+
+        assert_se(parse_size("111.4", 1000, &bytes) == 0);
+        assert_se(bytes == 111);
+
+        assert_se(parse_size(" 112 B", 1000, &bytes) == 0);
+        assert_se(bytes == 112);
+
+        assert_se(parse_size(" 112.6 B", 1000, &bytes) == 0);
+        assert_se(bytes == 112);
+
+        assert_se(parse_size("3.5 K", 1000, &bytes) == 0);
+        assert_se(bytes == 3*1000 + 500);
+
+        assert_se(parse_size("3. K", 1000, &bytes) == 0);
+        assert_se(bytes == 3*1000);
+
+        assert_se(parse_size("3.0 K", 1000, &bytes) == 0);
+        assert_se(bytes == 3*1000);
+
+        assert_se(parse_size("3. 0 K", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size(" 4 M 11.5K", 1000, &bytes) == 0);
+        assert_se(bytes == 4*1000*1000 + 11 * 1000 + 500);
+
+        assert_se(parse_size("3B3.5G", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size("3.5G3B", 1000, &bytes) == 0);
+        assert_se(bytes == 3ULL*1000*1000*1000 + 500*1000*1000 + 3);
+
+        assert_se(parse_size("3.5G 4B", 1000, &bytes) == 0);
+        assert_se(bytes == 3ULL*1000*1000*1000 + 500*1000*1000 + 4);
+
+        assert_se(parse_size("3B3G4T", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size("4T3G3B", 1000, &bytes) == 0);
+        assert_se(bytes == (4ULL*1000 + 3)*1000*1000*1000 + 3);
+
+        assert_se(parse_size(" 4 T 3 G 3 B", 1000, &bytes) == 0);
+        assert_se(bytes == (4ULL*1000 + 3)*1000*1000*1000 + 3);
+
+        assert_se(parse_size("12P", 1000, &bytes) == 0);
+        assert_se(bytes == 12ULL * 1000*1000*1000*1000*1000);
+
+        assert_se(parse_size("12P12P", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size("3E 2P", 1000, &bytes) == 0);
+        assert_se(bytes == (3 * 1000 + 2ULL) * 1000*1000*1000*1000*1000);
+
+        assert_se(parse_size("12X", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size("12.5X", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size("12.5e3", 1000, &bytes) == -EINVAL);
+
+        assert_se(parse_size("1000E", 1000, &bytes) == -ERANGE);
+        assert_se(parse_size("-1", 1000, &bytes) == -ERANGE);
+        assert_se(parse_size("-1000E", 1000, &bytes) == -ERANGE);
+
+        assert_se(parse_size("-1000P", 1000, &bytes) == -ERANGE);
+
+        assert_se(parse_size("-10B 20K", 1000, &bytes) == -ERANGE);
+}
+
+TEST(parse_range) {
         unsigned lower, upper;
 
         /* Successful cases */
@@ -347,7 +417,7 @@ static void test_parse_range(void) {
         assert_se(upper == 9999);
 }
 
-static void test_safe_atolli(void) {
+TEST(safe_atolli) {
         int r;
         long long l;
 
@@ -398,7 +468,7 @@ static void test_safe_atolli(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_safe_atou16(void) {
+TEST(safe_atou16) {
         int r;
         uint16_t l;
 
@@ -410,6 +480,14 @@ static void test_safe_atou16(void) {
         assert_se(r == 0);
         assert_se(l == 12345);
 
+        r = safe_atou16("+12345", &l);
+        assert_se(r == 0);
+        assert_se(l == 12345);
+
+        r = safe_atou16("  +12345", &l);
+        assert_se(r == 0);
+        assert_se(l == 12345);
+
         r = safe_atou16("123456", &l);
         assert_se(r == -ERANGE);
 
@@ -432,7 +510,7 @@ static void test_safe_atou16(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_safe_atoi16(void) {
+TEST(safe_atoi16) {
         int r;
         int16_t l;
 
@@ -444,6 +522,14 @@ static void test_safe_atoi16(void) {
         assert_se(r == 0);
         assert_se(l == -12345);
 
+        r = safe_atoi16("+12345", &l);
+        assert_se(r == 0);
+        assert_se(l == 12345);
+
+        r = safe_atoi16("  +12345", &l);
+        assert_se(r == 0);
+        assert_se(l == 12345);
+
         r = safe_atoi16("32767", &l);
         assert_se(r == 0);
         assert_se(l == 32767);
@@ -479,7 +565,7 @@ static void test_safe_atoi16(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_safe_atoux16(void) {
+TEST(safe_atoux16) {
         int r;
         uint16_t l;
 
@@ -524,7 +610,7 @@ static void test_safe_atoux16(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_safe_atou64(void) {
+TEST(safe_atou64) {
         int r;
         uint64_t l;
 
@@ -566,7 +652,7 @@ static void test_safe_atou64(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_safe_atoi64(void) {
+TEST(safe_atoi64) {
         int r;
         int64_t l;
 
@@ -613,7 +699,7 @@ static void test_safe_atoi64(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_safe_atoux64(void) {
+TEST(safe_atoux64) {
         int r;
         uint64_t l;
 
@@ -633,6 +719,22 @@ static void test_safe_atoux64(void) {
         assert_se(r == 0);
         assert_se(l == 11603985);
 
+        r = safe_atoux64("+12345", &l);
+        assert_se(r == 0);
+        assert_se(l == 0x12345);
+
+        r = safe_atoux64("  +12345", &l);
+        assert_se(r == 0);
+        assert_se(l == 0x12345);
+
+        r = safe_atoux64("+0x12345", &l);
+        assert_se(r == 0);
+        assert_se(l == 0x12345);
+
+        r = safe_atoux64("+0b11011", &l);
+        assert_se(r == 0);
+        assert_se(l == 11603985);
+
         r = safe_atoux64("0o11011", &l);
         assert_se(r == -EINVAL);
 
@@ -658,7 +760,7 @@ static void test_safe_atoux64(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_safe_atod(void) {
+TEST(safe_atod) {
         int r;
         double d;
         char *e;
@@ -715,7 +817,7 @@ static void test_safe_atod(void) {
         assert_se(r == -EINVAL);
 }
 
-static void test_parse_nice(void) {
+TEST(parse_nice) {
         int n;
 
         assert_se(parse_nice("0", &n) >= 0 && n == 0);
@@ -742,24 +844,7 @@ static void test_parse_nice(void) {
         assert_se(parse_nice("+20", &n) == -ERANGE);
 }
 
-static void test_parse_dev(void) {
-        dev_t dev;
-
-        assert_se(parse_dev("", &dev) == -EINVAL);
-        assert_se(parse_dev("junk", &dev) == -EINVAL);
-        assert_se(parse_dev("0", &dev) == -EINVAL);
-        assert_se(parse_dev("5", &dev) == -EINVAL);
-        assert_se(parse_dev("5:", &dev) == -EINVAL);
-        assert_se(parse_dev(":5", &dev) == -EINVAL);
-        assert_se(parse_dev("-1:-1", &dev) == -EINVAL);
-#if SIZEOF_DEV_T < 8
-        assert_se(parse_dev("4294967295:4294967295", &dev) == -EINVAL);
-#endif
-        assert_se(parse_dev("8:11", &dev) >= 0 && major(dev) == 8 && minor(dev) == 11);
-        assert_se(parse_dev("0:0", &dev) >= 0 && major(dev) == 0 && minor(dev) == 0);
-}
-
-static void test_parse_errno(void) {
+TEST(parse_errno) {
         assert_se(parse_errno("EILSEQ") == EILSEQ);
         assert_se(parse_errno("EINVAL") == EINVAL);
         assert_se(parse_errno("0") == 0);
@@ -780,7 +865,7 @@ static void test_parse_errno(void) {
         assert_se(parse_errno("EINVALaaa") == -EINVAL);
 }
 
-static void test_parse_mtu(void) {
+TEST(parse_mtu) {
         uint32_t mtu = 0;
 
         assert_se(parse_mtu(AF_UNSPEC, "1500", &mtu) >= 0 && mtu == 1500);
@@ -801,28 +886,28 @@ static void test_parse_mtu(void) {
         assert_se(parse_mtu(AF_UNSPEC, "", &mtu) == -EINVAL);
 }
 
-static void test_parse_loadavg_fixed_point(void) {
+TEST(parse_loadavg_fixed_point) {
         loadavg_t fp;
 
         assert_se(parse_loadavg_fixed_point("1.23", &fp) == 0);
-        assert_se(LOAD_INT(fp) == 1);
-        assert_se(LOAD_FRAC(fp) == 23);
+        assert_se(LOADAVG_INT_SIDE(fp) == 1);
+        assert_se(LOADAVG_DECIMAL_SIDE(fp) == 23);
 
         assert_se(parse_loadavg_fixed_point("1.80", &fp) == 0);
-        assert_se(LOAD_INT(fp) == 1);
-        assert_se(LOAD_FRAC(fp) == 80);
+        assert_se(LOADAVG_INT_SIDE(fp) == 1);
+        assert_se(LOADAVG_DECIMAL_SIDE(fp) == 80);
 
         assert_se(parse_loadavg_fixed_point("0.07", &fp) == 0);
-        assert_se(LOAD_INT(fp) == 0);
-        assert_se(LOAD_FRAC(fp) == 7);
+        assert_se(LOADAVG_INT_SIDE(fp) == 0);
+        assert_se(LOADAVG_DECIMAL_SIDE(fp) == 7);
 
         assert_se(parse_loadavg_fixed_point("0.00", &fp) == 0);
-        assert_se(LOAD_INT(fp) == 0);
-        assert_se(LOAD_FRAC(fp) == 0);
+        assert_se(LOADAVG_INT_SIDE(fp) == 0);
+        assert_se(LOADAVG_DECIMAL_SIDE(fp) == 0);
 
         assert_se(parse_loadavg_fixed_point("4096.57", &fp) == 0);
-        assert_se(LOAD_INT(fp) == 4096);
-        assert_se(LOAD_FRAC(fp) == 57);
+        assert_se(LOADAVG_INT_SIDE(fp) == 4096);
+        assert_se(LOADAVG_DECIMAL_SIDE(fp) == 57);
 
         /* Caps out at 2 digit fracs */
         assert_se(parse_loadavg_fixed_point("1.100", &fp) == -ERANGE);
@@ -837,28 +922,4 @@ static void test_parse_loadavg_fixed_point(void) {
         assert_se(parse_loadavg_fixed_point("", &fp) == -EINVAL);
 }
 
-int main(int argc, char *argv[]) {
-        log_parse_environment();
-        log_open();
-
-        test_parse_boolean();
-        test_parse_pid();
-        test_parse_mode();
-        test_parse_size();
-        test_parse_range();
-        test_safe_atolli();
-        test_safe_atou16();
-        test_safe_atoi16();
-        test_safe_atoux16();
-        test_safe_atou64();
-        test_safe_atoi64();
-        test_safe_atoux64();
-        test_safe_atod();
-        test_parse_nice();
-        test_parse_dev();
-        test_parse_errno();
-        test_parse_mtu();
-        test_parse_loadavg_fixed_point();
-
-        return 0;
-}
+DEFINE_TEST_MAIN(LOG_INFO);
